@@ -1419,6 +1419,9 @@ remove_prefixed_cb(void* key,
   auto const env = reinterpret_cast<char const*>(key);
   auto const prefix = reinterpret_cast<char const*>(user_data);
 
+  if (terminal_client_get_environment_prefix_filters_is_excluded(env))
+    return false;
+
   return g_str_has_prefix(env, prefix);
 }
 
@@ -1461,7 +1464,7 @@ terminal_screen_get_child_environment (TerminalScreen *screen,
   for (i = 0; filters[i]; ++i)
     g_hash_table_remove (env_table, filters[i]);
 
-  auto const pfilters = terminal_client_get_environment_filters ();
+  auto const pfilters = terminal_client_get_environment_prefix_filters ();
   for (i = 0; pfilters[i]; ++i) {
     g_hash_table_foreach_remove (env_table,
                                  GHRFunc(remove_prefixed_cb),
@@ -2271,7 +2274,7 @@ terminal_screen_has_foreground_process (TerminalScreen *screen,
   if (sysctl (mib, G_N_ELEMENTS (mib), nullptr, &len, nullptr, 0) == -1)
       return TRUE;
 
-  data_buf = g_malloc0 (len);
+  data_buf = (char*)g_malloc0 (len);
   if (sysctl (mib, G_N_ELEMENTS (mib), data_buf, &len, nullptr, 0) == -1)
       return TRUE;
   data = data_buf;
@@ -2283,7 +2286,7 @@ terminal_screen_has_foreground_process (TerminalScreen *screen,
   if (sysctl (mib, G_N_ELEMENTS (mib), nullptr, &len, nullptr, 0) == -1)
       return TRUE;
 
-  data_buf = g_malloc0 (len);
+  data_buf = (char*)g_malloc0 (len);
   if (sysctl (mib, G_N_ELEMENTS (mib), data_buf, &len, nullptr, 0) == -1)
       return TRUE;
   data = ((char**)data_buf)[0];
